import { Component, OnInit, DoCheck, ViewChild, Input } from '@angular/core';
import { Response } from '@angular/http';
import { Observable } from "rxjs/Observable";
import 'rxjs/add/operator/toPromise';

import { ComputeNode, ComputeNodeType, ParamsEquation, Nub, acSection, RegimeUniforme, MethodeResolution, CourbeRemousParams, CourbeRemous, cLog } from "jalhyd";

import { ParamService } from '../../services/param/param.service';
import { HttpService } from '../../services/http/http.service';
import { FormulaireService } from '../../services/formulaire/formulaire.service';
import { InternationalisationService } from '../../services/internationalisation/internationalisation.service';
import { FieldSet, InputField, SelectField, FormulaireDefinition, CalculatorType, Dependency, ExistenceDependency, ValueDependency } from '../../calculators/generic/formulaire';
import { NgParameter, ParamRadioConfig } from './ngparam';
import { CalculatorResultsComponent } from '../../components/calculator-results/calculator-results.component';
import { SectionResultsComponent } from '../../components/section-results/section-results.component';
import { RemousResultsComponent } from '../../components/remous-results/remous-results.component';
import { Observer } from '../../services/observer';

@Component({
    selector: 'hydrocalc',
    templateUrl: "./calculator.component.html",
    styles: [`
    .button_compute {
        height: 3em;
        width: 30%;
    }
    `
    ]
})
export class GenericCalculatorComponent implements OnInit, DoCheck, Observer {
    /**
     * Calculator type input attribute
     */
    private _calculatorType: CalculatorType;

    @Input()
    private set type(s: string) {
        this._calculatorType = CalculatorType[s];
    }

    /**
     * type de noeud de calcul actuel (utilisé pour les sections, à sortir de ce composant générique)
     */
    private _nodeType: ComputeNodeType;

    /**
     * composant d'affichage des résultats
     */
    @ViewChild(CalculatorResultsComponent)
    private resultsComponent: CalculatorResultsComponent;

    /**
     * composant d'affichage des résultats des section paramétrées
     */
    @ViewChild(SectionResultsComponent)
    private sectionResultsComponent: SectionResultsComponent;

    /**
     * composant d'affichage des résultats des section paramétrées
     */
    @ViewChild(RemousResultsComponent)
    private remousResultsComponent: RemousResultsComponent;

    /**
     * objet JSON chargé depuis le fichier de traduction
     */
    private _localisation = {};

    private _formulaire: FormulaireDefinition;

    /**
     * flag d'affichage du composant de résultats fixes/variables
     */
    private _showResultsFixVar: boolean = false;

    /**
     * flag d'affichage du composant de résultats du calcul de section
     */
    private _showResultsSection: boolean = false;

    /**
     * flag d'affichage du composant de résultats du calcul des courbes de remous
     */
    private _showResultsRemous: boolean = false;

    constructor(private paramService: ParamService, private httpService: HttpService, private intlService: InternationalisationService, private formulaireService: FormulaireService) {
        this.intlService.addObserver(this);
    }

    private get fieldSets(): FieldSet[] {
        if (this._formulaire == undefined)
            return [];
        return this._formulaire.getFieldSets(this._nodeType);
    }

    private loadLocalisation(): Promise<string> {
        let ths = this;
        let processData = function (s: string) {
            // fermeture nécessaire pour capturer la valeur de this (undefined sinon)
            ths._localisation = JSON.parse(s);
        }

        let f: string = this.formulaireService.getConfigPathPrefix(this._calculatorType) + this.intlService.currentLanguage.tag + ".json"
        let resp: Observable<Response> = this.httpService.httpGetRequestResponse(undefined, undefined, undefined, f);

        let prom = resp.map(res => res.text()).toPromise();
        prom.then((res) => {
            processData(res);
        })

        return prom;
    }

    private loadFormulaire(): Promise<FormulaireDefinition> {
        if (this._formulaire == undefined) {
            let prom = this.formulaireService.loadFormulaire(this._calculatorType);
            prom.then(res => {
                this._formulaire = res;
            });
            prom.then(
                _ => this.loadLocalisation()
            );
            prom.then(
                _ => this.formulaireService.updateLocalisation(this._localisation)
            );

            return prom;
        }
        return undefined;
    }

    /*
    private getHtmlElementFromId_helper(elm: HTMLElement, id: string): HTMLElement {
        if (elm.hasAttribute('id'))
            if (elm.getAttribute('id') === id)
                return elm;

        let kids: NodeList = elm.childNodes;
        for (let i = 0; i < kids.length; i++) {
            let kid = kids[i];
            if (kid instanceof HTMLElement) {
                let e = this.getHtmlElementFromId_helper(<HTMLElement>kid, id);
                if (e != undefined)
                    return e;
            }
        }

        return undefined;
    }

    private getHtmlElementFromId(id: string): HTMLElement {
        let he: HTMLElement = this.elementRef.nativeElement;
        return this.getHtmlElementFromId_helper(he, id);
    }

    private getHtmlElementValue(e: HTMLElement): string {
        if (e instanceof HTMLSelectElement)
            return e.value;
        return undefined;
    }
    private setHtmlElementValue(e: HTMLElement, val: string) {
        if (e instanceof HTMLInputElement)
            e.value = val;
    }
    */

    private applyDependencies() {
        if (this._formulaire == undefined)
            return;
        this._formulaire.applyDependencies();

        /*
        for (let d of this._formulaire.dependencies) {
            if (d instanceof ExistenceDependency) {
            }
            else if (d instanceof ValueDependency) {
                let vd = <ValueDependency>d;
                let master: HTMLElement = this.getHtmlElementFromId(d.masterElement.id);
                if (this.getHtmlElementValue(master) == vd.masterValue) {
                    let slave: HTMLElement = this.getHtmlElementFromId(d.slaveElement.id);
                    this.setHtmlElementValue(slave, vd.slaveValue);
                }
            }
        }
        */
    }

    ngOnInit() {
        let prom = this.loadFormulaire();
        prom.then(
            _ => {
                let ssf: SelectField = <SelectField>this._formulaire.getFormulaireElementById("select_section");
                this.updateSectionType(ssf.getValue());

                this.applyDependencies();
            }
        );
    }

    ngDoCheck() {
        // let q = this.getParamFromSymbol("Q");  // A VIRER !!!!
        // if (q != undefined) {
        //     q.radioState = ParamRadioConfig.VAR;
        //     q.minValue = 1.5;
        //     q.maxValue = 6;
        //     q.stepValue = 0.3;
        // }
    }

    /*
     * gestion des événements clic sur les radios :
     * envoi d'un message au composant parent
     * cf. https://angular.io/guide/component-interaction#parent-listens-for-child-event
     */

    private onRadioClick(info: string) {
        this._showResultsFixVar = false;
        this._showResultsSection = false;
        this._showResultsRemous = false;

        let tmp: string[] = info.split("_");
        let symbol: string = tmp[0];

        let sourceParam = this._formulaire.getParamFromSymbol(symbol);
        let oldState: ParamRadioConfig = sourceParam.radioState;
        let newState: ParamRadioConfig = ParamRadioConfig[tmp[1].toUpperCase()];

        switch (oldState) {
            case ParamRadioConfig.FIX:
                switch (newState) {
                    case ParamRadioConfig.VAR:
                        this._formulaire.resetOther(sourceParam, ParamRadioConfig.CAL);
                        break;

                    case ParamRadioConfig.CAL:
                        this._formulaire.resetOther(sourceParam, ParamRadioConfig.VAR);
                        break;
                }
                break;

            case ParamRadioConfig.VAR:
                switch (newState) {
                    case ParamRadioConfig.CAL:
                        this._formulaire.resetOther(sourceParam, ParamRadioConfig.VAR);
                        break;
                }
                break;

            case ParamRadioConfig.CAL:
                switch (newState) {
                    case ParamRadioConfig.FIX:
                        this._formulaire.setDefault();
                        break;

                    case ParamRadioConfig.VAR:
                        this._formulaire.resetOther(sourceParam, ParamRadioConfig.CAL);
                        this._formulaire.setDefault();
                        break;
                }
        }

        sourceParam.radioState = newState;

        // this.appRef.tick();
        //        this.changeDetectorRef.detectChanges();  // provoque une détection des changements dans les contrôles

        this.resultsComponent.reset(false);
        this.applyDependencies();
    }

    protected getParameterValue(symbol: string): number {
        return this._formulaire.getParameterValue(symbol);
    }

    protected getNodeParameterValue(nodeType: ComputeNodeType, symbol: string): number {
        return this._formulaire.getNodeParameterValue(nodeType, symbol);
    }

    private getComputedParameter(): NgParameter {
        return this._formulaire.getParamFromState(ParamRadioConfig.CAL);
    }

    private getVariatedParameter(): NgParameter {
        return this._formulaire.getParamFromState(ParamRadioConfig.VAR);
    }

    private isMySectionType(nodeType: ComputeNodeType, strict: boolean) {
        let res: boolean = nodeType == this._nodeType;
        if (strict)
            return res;

        return res || nodeType == ComputeNodeType.SectionParametree
    }

    private getSectionVariatedParameter(): NgParameter {
        let res: NgParameter = this._formulaire.getParamFromState(ParamRadioConfig.VAR);
        if (res != undefined && this.isMySectionType(res.computeNodeType, false))
            return res;
        return undefined;
    }

    private addFixedResults(nDigits: number, displaySymbol: boolean) {
        for (let p of this._formulaire.getInputParameters())
            if (p.radioState == ParamRadioConfig.FIX && p.symbol !== "Pr")
                this.resultsComponent.addFixedResult(p, p.getValue(), nDigits, displaySymbol);
    }

    private addSectionFixedResults(nDigits: number, displaySymbol: boolean) {
        for (let p of this._formulaire.getDisplayedInputParameters())
            if (p.radioState == ParamRadioConfig.FIX && this.isMySectionType(p.computeNodeType, false) && p.symbol !== "Pr")
                this.resultsComponent.addFixedResult(p, p.getValue(), nDigits, displaySymbol);
    }

    private addSectionFixedResult(val: number, label: string, nDigits: number, drawLabel: string = undefined) {
        this.sectionResultsComponent.addResult(val, label, nDigits, drawLabel);
    }

    private getSectionComputedParam(): { symbol: string, label: string } {
        let targetSelect: SelectField = <SelectField>this._formulaire.getFieldById("select_target");
        let targetValue: string = targetSelect.getValue();

        let prefix = "select_target_"
        let l = prefix.length;
        let symbol: string = targetValue.substr(l, targetValue.length - l);

        let label = targetSelect.getLabel();
        return { symbol, label };
    }

    private doComputeSectionVar(varParam: NgParameter) {
        let prec: number = this.getParameterValue("Pr"); // précision
        let nDigits = -Math.log10(prec);

        this.resultsComponent.reset(false);
        this.addSectionFixedResults(nDigits, false);

        let computedParam = this.getSectionComputedParam();

        this.resultsComponent.setVariableParamHeaderFromParameter(varParam, false);
        this.resultsComponent.setVariableResultHeader(computedParam["label"]);

        let ssf: SelectField = <SelectField>this._formulaire.getFormulaireElementById("select_section");
        let typeSect: ComputeNodeType = FormulaireDefinition.getComputeNodeTypeFromSection(ssf.getValue(), CalculatorType.SectionParametree);

        var np: [acSection, ParamsEquation] = this.formulaireService.getSectionNubAndParameters(CalculatorType.SectionParametree, typeSect);
        let sect: acSection = np[0];
        let prms: ParamsEquation = np[1];

        let min: number = +varParam.minValue;
        let max: number = +varParam.maxValue;
        let step: number = +varParam.stepValue;

        let yField: InputField = <InputField>this._formulaire.getFormulaireElementById("Y");
        let Y: number = yField.getValue();

        let compSymbol = computedParam["symbol"];
        for (let val = min; val <= max; val += step) {
            prms[varParam.symbol].v = val;

            sect.Reset(true);
            let res = sect.Calc(compSymbol, Y);
            this.resultsComponent.addVarResult(val, res, nDigits);
        }

        this.resultsComponent.setGraphTitle(computedParam.symbol + " = f( " + varParam.symbol + " )");
        this.resultsComponent.generateGraph();
        this._showResultsFixVar = true;
    }

    private doComputeSection() {
        this.sectionResultsComponent.reset();

        let varParam = this.getSectionVariatedParameter();
        if (varParam != undefined) {
            this.doComputeSectionVar(varParam);
            return;
        }

        let ssf: SelectField = <SelectField>this._formulaire.getFormulaireElementById("select_section");
        let typeSect: ComputeNodeType = FormulaireDefinition.getComputeNodeTypeFromSection(ssf.getValue(), CalculatorType.SectionParametree);
        var np: [acSection, ParamsEquation] = this.formulaireService.getSectionNubAndParameters(CalculatorType.SectionParametree, typeSect);

        let sect: acSection = np[0];
        let prms: ParamsEquation = np[1];

        let prec: number = this.getNodeParameterValue(ComputeNodeType.SectionParametree, "Pr"); // précision
        let nDigits = -Math.log10(prec);

        let Y = prms.map.Y.v; // tirant d'eau original (doit être fourni à acSection.Calc() sous peine d'être modifié par les appels successifs car c'est en même temps un paramètre et une variable temporaire)

        // charge spécifique
        let Hs = sect.Calc("Hs", Y);
        this.addSectionFixedResult(Hs, "La charge spécifique (m)", nDigits, "Hs");

        // charge critique
        let Hsc = sect.Calc("Hsc", Y);
        this.addSectionFixedResult(Hsc, "La charge critique (m)", nDigits, "Hsc");

        // largeur au miroir
        let B = sect.Calc("B", Y);
        this.addSectionFixedResult(B, "La largeur au miroir (m)", nDigits);

        // périmètre hydraulique
        let P = sect.Calc("P", Y);
        this.addSectionFixedResult(P, "Le périmètre mouillé (m)", nDigits);

        // surface hydraulique
        let S = sect.Calc("S", Y);
        this.addSectionFixedResult(S, "La surface mouillée (m2)", nDigits);

        // rayon hydraulique
        let R = sect.Calc("R", Y);
        this.addSectionFixedResult(R, "Le rayon hydraulique (m)", nDigits);

        // vitesse moyenne
        let V = sect.Calc("V", Y);
        this.addSectionFixedResult(V, "La vitesse moyenne (m/s)", nDigits);

        // nombre de Froude
        let Fr = sect.Calc("Fr", Y);
        this.addSectionFixedResult(Fr, "Le Froude", nDigits);

        // tirant d'eau critique
        let Yc = sect.Calc("Yc", Y);
        this.addSectionFixedResult(Yc, "Le tirant d'eau critique (m)", nDigits, "Yc");

        // tirant d'eau normal
        let Yn = sect.Calc("Yn", Y);
        this.addSectionFixedResult(Yn, "Le tirant d'eau normal (m)", nDigits, "Yn");

        // tirant d'eau fluvial
        let Yf = sect.Calc("Yf", Y);
        this.addSectionFixedResult(Yf, "Le tirant d'eau fluvial (m)", nDigits, "Yf");

        // tirant d'eau torrentiel
        let Yt = sect.Calc("Yt", Y);
        this.addSectionFixedResult(Yt, "Le tirant d'eau torrentiel (m)", nDigits, "Yt");

        // tirant d'eau conjugué
        let Yco = sect.Calc("Yco", Y);
        this.addSectionFixedResult(Yco, "Le tirant d'eau conjugué (m)", nDigits, "Yco");

        // perte de charge
        let J = sect.Calc("J", Y);
        this.addSectionFixedResult(J, "La perte de charge (m/m)", nDigits);

        // Variation linéaire de l'énergie spécifique
        let IJ = sect.Calc("I-J", Y);
        this.addSectionFixedResult(IJ, "Variation linéaire de l'énergie spécifique (m/m)", nDigits);

        // impulsion hydraulique
        let Imp = sect.Calc("Imp", Y);
        this.addSectionFixedResult(Imp, "Impulsion (m/m)", nDigits);

        // contrainte de cisaillement
        let Tau0 = sect.Calc("Tau0", Y);
        this.addSectionFixedResult(Tau0, "La force tractrice (N)", nDigits);

        this.sectionResultsComponent.section = sect;
        this._showResultsSection = true;
    }

    private addRemousResult(x: string, f: number, t: number, nDigits: number) {
        let flu = f == undefined ? "" : f.toFixed(nDigits);
        let tor = t == undefined ? "" : f.toFixed(nDigits);
        this.remousResultsComponent.addResult(x, flu, tor);
    }

    private doComputeRemous() {
        this.remousResultsComponent.reset();

        let ssf: SelectField = <SelectField>this._formulaire.getFormulaireElementById("select_section");
        let typeSect: ComputeNodeType = FormulaireDefinition.getComputeNodeTypeFromSection(ssf.getValue(), CalculatorType.CourbeRemous);
        var np: [acSection, ParamsEquation] = this.formulaireService.getSectionNubAndParameters(CalculatorType.CourbeRemous, typeSect, false);

        let sect: acSection = np[0];
        let prmSect: ParamsEquation = np[1];

        let prec: number = this.getNodeParameterValue(ComputeNodeType.CourbeRemous, "Pr"); // précision
        let nDigits: number = -Math.log10(prec);

        let Yamont: number = this.getNodeParameterValue(ComputeNodeType.CourbeRemous, "Yamont"); // tirant amont
        let Yaval: number = this.getNodeParameterValue(ComputeNodeType.CourbeRemous, "Yaval"); // tirant aval
        let Dx: number = this.getNodeParameterValue(ComputeNodeType.CourbeRemous, "Dx"); // pas de discrétisation
        let Long: number = this.getNodeParameterValue(ComputeNodeType.CourbeRemous, "Long"); // longueur du bief
        let If: number = this.getNodeParameterValue(ComputeNodeType.CourbeRemous, "If"); // pente du fond
        let YB: number = this.getNodeParameterValue(ComputeNodeType.CourbeRemous, "YB"); // hauteur de berge
        let Yn: number = sect.Calc("Yn"); // hauteur normale
        let Yc: number = sect.Calc("Yc"); // hauteur critique

        // méthode de résolution

        let msf: SelectField = <SelectField>this._formulaire.getFormulaireElementById("select_resolution");
        let smeth: string = msf.getValue();
        let methRes: MethodeResolution;
        if (smeth == "select_resolution_trap")
            methRes = MethodeResolution.Trapezes;
        else if (smeth == "select_resolution_rk4")
            methRes = MethodeResolution.RungeKutta4;
        else if (smeth == "select_resolution_euler")
            methRes = MethodeResolution.EulerExplicite;
        else
            throw "GenericCalculatorComponent.doComputeRemous() : type de méthode de résolution '" + smeth + "' inconnu";

        // calcul

        let prmCR: CourbeRemousParams = new CourbeRemousParams(sect, Yamont, Yaval, Long, Dx, methRes);
        let log: cLog = new cLog();
        let cr = new CourbeRemous(prmCR, log);
        let res = cr.calculRemous(undefined);

        // affichage du graphe

        this.remousResultsComponent.setPenteFond(If);
        this.remousResultsComponent.setLongBief(Long);
        this.remousResultsComponent.setHauteurBerge(YB);
        this.remousResultsComponent.setHauteurNormale(Yn);
        this.remousResultsComponent.setHauteurCritique(Yc);

        // affichage du journal

        for (let l of log.messages)
            this.remousResultsComponent.addLogEntry(l.toString());

        // affichage des resultats numériques

        let kFlu = Object.keys(res.flu);
        let kTor = Object.keys(res.tor);
        for (let i = 0; i < res.trX.length; i++) {
            let x: string = res.trX[i];
            // let f = i < kFlu.length ? res.flu[kFlu[i]] : undefined;
            // let t = i < kTor.length ? res.tor[kTor[i]] : undefined;
            let f = res.flu[x];
            let t = res.tor[x];
            this.addRemousResult(x, f, t, nDigits);
        }

        this.remousResultsComponent.generateGraph();
        this._showResultsRemous = true;
    }

    private doCompute() {
        this._showResultsFixVar = false;
        this._showResultsSection = false;
        this._showResultsRemous = false;

        if (this._calculatorType == CalculatorType.SectionParametree) {
            this.doComputeSection();
            return;
        }

        if (this._calculatorType == CalculatorType.CourbeRemous) {
            this.doComputeRemous();
            return;
        }

        let np: [Nub, ParamsEquation];
        let nub: Nub;
        let prms: ParamsEquation;
        let rg: boolean = this._calculatorType == CalculatorType.RegimeUniforme;
        if (rg) {
            let snp: [acSection, ParamsEquation] = this.formulaireService.getSectionNubAndParameters(CalculatorType.RegimeUniforme, this._nodeType);
            nub = new RegimeUniforme(snp[0]);
            prms = snp[1];
        }
        else {
            np = this.formulaireService.getNubAndParameters(this._calculatorType);
            nub = np[0];
            prms = np[1];
        }

        let prec: number = this.getParameterValue("Pr"); // précision
        let nDigits: number = -Math.log10(prec);

        let computedParam: NgParameter = this.getComputedParameter();

        let varParam: NgParameter = this.getVariatedParameter();
        this.resultsComponent.reset(varParam == undefined);
        if (varParam == undefined) {
            // pas de paramètre à varier

            let res = nub.Calc(computedParam.symbol, 0, prec).vCalc;

            this.addFixedResults(nDigits, !rg);
            this.resultsComponent.addFixedResult(computedParam, res, nDigits, !rg);
        }
        else {
            // il y a un paramètre à varier

            this.addFixedResults(nDigits, !rg);
            this.resultsComponent.setVariableParamHeaderFromParameter(varParam, !rg);
            this.resultsComponent.setVariableResultHeaderFromParameter(computedParam);

            let min: number = +varParam.minValue;
            let max: number = +varParam.maxValue;
            let step: number = +varParam.stepValue;

            for (let val = min; val <= max; val += step) {
                prms[varParam.symbol].v = val;

                let res = nub.Calc(computedParam.symbol, 0, prec).vCalc;
                this.resultsComponent.addVarResult(val, res, nDigits);
            }

            this.resultsComponent.setGraphTitle(computedParam.symbol + " = f( " + varParam.symbol + " )");
            this.resultsComponent.generateGraph();
        }
        this._showResultsFixVar = true;
    }

    private getResultsStyleDisplay() {
        return this._showResultsFixVar ? "block" : "none";
    }

    private getSectionResultsStyleDisplay() {
        return this._showResultsSection ? "block" : "none";
    }

    private getRemousResultsStyleDisplay() {
        return this._showResultsRemous ? "block" : "none";
    }

    private getFieldsetStyleDisplay(id: string) {
        let isDisplayed: boolean = this._formulaire.isDisplayed(id);
        return isDisplayed ? "block" : "none";
    }

    // interface Observer

    update(data: any): void {
        // message de InternationalisationService
        const promise = this.loadLocalisation()
            .then(() => {
                this.formulaireService.updateLocalisation(this._localisation);
            });
    }

    private updateSectionType(val: string) {
        let select_prefix: string = "select_section_";
        //TODO ici on fait un cas particulier, il faudra sortir ça
        if (val.startsWith(select_prefix)) {
            let l = select_prefix.length;
            let type: string = val.substr(l, val.length - l);

            switch (this._calculatorType) {
                case CalculatorType.SectionParametree:
                    switch (type) {
                        case "trapez":
                            this._nodeType = ComputeNodeType.SectionTrapeze;
                            break;

                        case "rect":
                            this._nodeType = ComputeNodeType.SectionRectangle;
                            break;

                        case "circ":
                            this._nodeType = ComputeNodeType.SectionCercle;
                            break;

                        case "puiss":
                            this._nodeType = ComputeNodeType.SectionPuissance;
                            break;
                    }
                    break;

                case CalculatorType.RegimeUniforme:
                    switch (type) {
                        case "trapez":
                            this._nodeType = ComputeNodeType.RegimeUniformeTrapeze;
                            break;

                        case "rect":
                            this._nodeType = ComputeNodeType.RegimeUniformeRectangle;
                            break;

                        case "circ":
                            this._nodeType = ComputeNodeType.RegimeUniformeCercle;
                            break;

                        case "puiss":
                            this._nodeType = ComputeNodeType.RegimeUniformePuissance;
                            break;
                    }
                    break;

                case CalculatorType.CourbeRemous:
                    switch (type) {
                        case "trapez":
                            this._nodeType = ComputeNodeType.CourbeRemousTrapeze;
                            break;

                        case "rect":
                            this._nodeType = ComputeNodeType.CourbeRemousRectangle;
                            break;

                        case "circ":
                            this._nodeType = ComputeNodeType.CourbeRemousCercle;
                            break;

                        case "puiss":
                            this._nodeType = ComputeNodeType.CourbeRemousPuissance;
                            break;
                    }
                    break;

                default:
                    throw "GenericCalculatorComponent.updateSectionType() : section " + val + " / type de calculette " + CalculatorType[this._calculatorType] + " non pris en charge"
            }
        }
    }

    /**
     * réception d'un événement d'un select
     */
    private onSelectChanged(val: string) {
        this.resultsComponent.reset(false);
        this.sectionResultsComponent.reset();
        this.updateSectionType(val);
        this.applyDependencies();
    }
}
