import { ComputeNodeType, ParamDefinition } from 'jalhyd';

import { NgParameter, ParamRadioConfig } from '../generic/ngparam';
import { ParamService } from '../../services/param/param.service';
import { StringMap } from '../../stringmap';
import { logObject } from '../../util';

export enum CalculatorType {
    ConduiteDistributrice, LechaptCalmon, SectionParametree, RegimeUniforme, CourbeRemous
}

export class FormulaireDefinition {
    /**
     * objet JSON chargé depuis le fichier de configuration de la calculette
     */
    private _config = {};

    /**
     * symbole du paramètre à calculer par défaut (cf config "idCal")
     */
    private _defaultCalculatedParam: string;

    private _fieldSets: FieldSet[] = [];

    private _dependencies: Dependency[] = [];

    /**
     * type de noeud de calcul par défaut
     */
    private _defaultNodeType: ComputeNodeType;

    constructor(private paramService: ParamService, private _type: CalculatorType) {
    }

    public get type(): CalculatorType {
        return this._type;
    }

    public getFieldSets(nodeType: ComputeNodeType): FieldSet[] {
        let res: FieldSet[] = [];

        for (let fs of this._fieldSets)
            if (fs.computeNodeType == nodeType || fs.computeNodeType == this._defaultNodeType)
                res.push(fs);

        return res;
    }

    public get dependencies(): Dependency[] {
        return this._dependencies;
    }

    private getFieldSet(id: string) {
        for (let fs of this._fieldSets) {
            if (fs.id == id)
                return fs;
        }
        return undefined;
    }

    public getParamFromSymbol(symbol: string): NgParameter {
        for (let fs of this._fieldSets) {
            for (let p of fs.fields) {
                if (p instanceof NgParameter)
                    if (p.symbol === symbol)
                        return p;
            }
        }
        return undefined;
    }

    public getParamFromState(st: ParamRadioConfig): NgParameter {
        for (let fs of this._fieldSets) {
            for (let p of fs.fields) {
                if (p instanceof NgParameter)
                    if (p.radioState == st)
                        return p;
            }
        }
        return undefined;
    }


    /**
     * remet tous les paramètres à FIX sauf "me" et ceux (celui) à l'état "except"
     */
    public resetOther(me: NgParameter, except: ParamRadioConfig) {
        for (let fs of this._fieldSets) {
            for (let p of fs.fields) {
                if (p instanceof NgParameter)
                    if (p != me && p.radioState != except && p.radioConfig != ParamRadioConfig.FIX)
                        p.radioState = ParamRadioConfig.FIX;
            }
        }
    }

    /**
     * met le paramètre par défaut à CAL
     */
    public setDefault() {
        let defaultParamCal = this.getParamFromSymbol(this._defaultCalculatedParam);
        defaultParamCal.radioState = ParamRadioConfig.CAL;
    }

    public getInputParameters(): NgParameter[] {
        let res = [];
        for (let fs of this._fieldSets)
            for (let p of fs.fields)
                if (p instanceof NgParameter)
                    res.push(p);
        return res;
    }

    public getDisplayedInputParameters(): NgParameter[] {
        let res = [];
        for (let fs of this._fieldSets)
            if (fs.isDisplayed)
                for (let p of fs.fields)
                    if (p instanceof NgParameter && p.isDisplayed)
                        res.push(p);
        return res;
    }

    public getNodeParameterValue(nodeType: ComputeNodeType, symbol: string): number {
        for (let fs of this._fieldSets) {
            for (let p of fs.fields) {
                if (p instanceof NgParameter)
                    // if (p.computeNodeType == nodeType && (p.symbol === symbol || p.alias == symbol)) {
                    if (p.computeNodeType == nodeType && p.symbol === symbol) {
                        switch (p.radioState) {
                            case ParamRadioConfig.FIX:
                                return p.getValue();

                            case ParamRadioConfig.VAR:
                            case ParamRadioConfig.CAL:
                                return undefined;
                        }
                    }
            }
        }

        if (nodeType != this._defaultNodeType)
            return this.getNodeParameterValue(this._defaultNodeType, symbol);

        throw "Formulaire.getNodeParameterValue() : pas de paramètre " + symbol + " trouvé pour le noeud " + ComputeNodeType[nodeType] + "(" + nodeType + ")";
    }

    public getParameterValue(symbol: string): number {
        return this.getNodeParameterValue(this._defaultNodeType, symbol);
    }

    public getFormulaireElementById(id: string): FormulaireElement {
        for (let fs of this._fieldSets) {
            if (fs.id == id)
                return fs;

            for (let p of fs.fields)
                if (p.id === id)
                    return p;
        }
        return undefined;
    }

    public getFieldById(id: string): Field {
        let res = this.getFormulaireElementById(id);
        if (res instanceof Field)
            return res;
        return undefined;
    }

    private parse_value_dependencies(json: {}, slave: FormulaireElement) {
        for (let di in json) {
            let d = json[di];
            let masterField: FormulaireElement = this.getFormulaireElementById(d["refid"]);
            if (masterField != undefined) {
                let masterValue = d["refvalue"];
                let dep = new ValueDependency(masterField, slave, masterValue);
                dep.slaveValue = d["value"];
                this._dependencies.push(dep);
            }
        }
    }

    private parse_existence_dependencies(json: {}, slave: FormulaireElement) {
        for (let di in json) {
            let d = json[di];
            let masterField: FormulaireElement = this.getFormulaireElementById(d["refid"]);
            if (masterField != undefined) {
                let rv = d["refvalue"];
                if (rv != undefined)
                    var mc: DependencyCondition = new ValueDependencyCondition(rv);
                else {
                    let cond = d["cond"];
                    if (cond != undefined) {
                        switch (cond) {
                            case "isvar":
                                var mc = new DependencyCondition(DependencyConditionType.IsVariable);
                                break;

                            case "isdisp":
                                var mc = new DependencyCondition(DependencyConditionType.IsDisplayed);
                                break;

                            default:
                                throw "Formulaire.parse_existence_dependencies() : type de condition '" + cond + "' non pris en charge";
                        }
                    }
                    else
                        throw "Formulaire.parse_existence_dependencies() : infos de dépendance manquantes/non prises en charge";
                }
                let dep = new ExistenceDependency(masterField, slave, mc);
                this._dependencies.push(dep);
            }
        }
    }

    private parse_dependencies(slave: FormulaireElement, json: {}) {
        let dep = json["dep_value"];
        if (dep != undefined) {
            this.parse_value_dependencies(dep, slave);
        }
        else {
            dep = json["dep_exist"];
            if (dep != undefined) {
                this.parse_existence_dependencies(dep, slave);
            }
        }
    }

    private parse_check(node_type: ComputeNodeType, field: {}): CheckField {
        let id = field["id"];
        let res: CheckField = new CheckField(node_type, id);
        let value = field["value"];

        res.setValue(value == "true");

        this.parse_dependencies(res, field);

        return res;
    }

    private parse_select(node_type: ComputeNodeType, field: {}): SelectField {
        let id = field["id"];
        let res: SelectField = new SelectField(node_type, id);
        let values = field["select"];

        for (let v of values) {
            let e: SelectEntry = new SelectEntry(v["id"], undefined);
            res.addEntry(e);
        }

        this.parse_dependencies(res, field);

        return res;
    }

    private parse_input(node_type: ComputeNodeType, fieldset: {}, field: {}): NgParameter {
        let input_id: string = field["id"];
        let res: NgParameter = this.paramService.getParameter(node_type, input_id);
        if (res == undefined)
            throw "pas de paramètre '" + input_id + "' trouvé dans le noeud " + ComputeNodeType[node_type] + "(" + node_type + ") ou " + ComputeNodeType[this._defaultNodeType] + "(" + this._defaultNodeType + ")";

        res.unit = field["unit"];
        let val = field["value"];
        if (val != undefined)
            res.setValue(+val);
        res.radioConfig = NgParameter.getRadioConfig(fieldset["option"]);
        res.radioState = ParamRadioConfig.FIX;
        res.isDefault = false; // malgré le fait qu'il soit initialisé dans la déclaration de la classe NgParam à false, quand on relit sa valeur, il vaut undefined (merci Microsoft)

        this.parse_dependencies(res, field);

        return res;
    }

    private parse_fieldset(fieldset: {}, conf_id: string) {
        let nt: string = fieldset["nodeType"];
        let node_type: ComputeNodeType;
        node_type = nt == undefined ? this._defaultNodeType : ComputeNodeType[nt];

        let res: FieldSet = new FieldSet(node_type, conf_id);
        this._fieldSets.push(res);

        let fields = fieldset["fields"];
        for (let field_index in fields) {
            let field = fields[field_index];
            if (field["type"] === "input") {
                let param = this.parse_input(node_type, fieldset, field);
                if (param != undefined)
                    res.addField(param);
            } else if (field["type"] === "select") {
                let param = this.parse_select(node_type, field);
                res.addField(param);
            } else if (field["type"] === "check") {
                let param = this.parse_check(node_type, field);
                res.addField(param);
            }
        }

        this.parse_dependencies(res, fieldset);
    }

    private getOption(option: string): string {
        for (let conf_index in this._config) {
            let conf = this._config[conf_index];
            if (conf["id"] === "options")
                return conf[option];
        }

        return undefined;
    }

    private parseNodeType() {
        let nt: string = this.getOption("nodeType");
        if (nt == undefined)
            throw "l'option obligatoire 'nodeType' est absente du fichier de définition de formulaire";
        this._defaultNodeType = ComputeNodeType[nt];
    }

    public parseConfig(config: {}) {
        this._config = config;
        this._dependencies = [];
        this._defaultCalculatedParam = undefined;

        this.parseNodeType();

        for (let conf_index in this._config) {
            let conf = this._config[conf_index];
            let conf_id: string = conf["id"];

            // field set
            if (conf_id.startsWith("fs_")) {
                this.parse_fieldset(conf, conf_id);
            }
            // options globales
            else if (conf_id === "options") {
                // id du paramètre à calculer par défaut
                this._defaultCalculatedParam = conf["idCal"];
                if (this._defaultCalculatedParam != undefined) {
                    let p = this.getParamFromSymbol(this._defaultCalculatedParam);
                    p.isDefault = true;
                    p.radioState = ParamRadioConfig.CAL;
                }
            }
        }

        // logObject(this._fieldSets, "fieldsets");
        // logObject(this._dependencies, "dependences");
    }

    private getDependencyFromMasterValue(v: any): Dependency {
        for (let d of this._dependencies)
            if (d.masterCondition.type == DependencyConditionType.HasValue) {
                let mv = (<ValueDependencyCondition>d.masterCondition).value;
                if (mv === v)
                    return d;
            }
        return undefined;
    }

    private isNumber(s: string): boolean {
        return Number(s) != NaN;
    }

    private prepareExistenceDependencies() {
        // si des FormulaireElement sont présents dans des ExistenceDependency, on met leur membre isDisplayed à false

        for (let d of this.dependencies)
            if (d instanceof ExistenceDependency) {
                let slave: FormulaireElement = this.getFormulaireElementById(d.slaveElement.id);
                slave.isDisplayed = false;
            }
    }

    public applyDependencies() {
        this.prepareExistenceDependencies();

        for (let d of this.dependencies) {
            let master: FormulaireElement = this.getFormulaireElementById(d.masterElement.id);
            if (d instanceof ExistenceDependency) {
                let slave: FormulaireElement = this.getFormulaireElementById(d.slaveElement.id);
                slave.isDisplayed = slave.isDisplayed || master.verifiesDependency(d);
            }
            else if (d instanceof ValueDependency) {
                let vd = <ValueDependency>d;
                /*
                let master: HTMLElement = this.getHtmlElementFromId(d.masterElement.id);
                if (this.getHtmlElementValue(master) == vd.masterValue) {
                    let slave: HTMLElement = this.getHtmlElementFromId(d.slaveElement.id);
                    this.setHtmlElementValue(slave, vd.slaveValue);
                }
                */
                if (master.verifiesDependency(d)) {
                    let slave = this.getFieldById(d.slaveElement.id);
                    if (this.isNumber(vd.slaveValue))
                        slave.setValue(+vd.slaveValue);
                    else
                        slave.setValue(vd.slaveValue);
                }
            }
        }
    }

    public printDependencies() {
        for (let d of this.dependencies)
            console.log(d.toString());
    }

    public static getComputeNodeTypeFromSection(sect: string, calc: CalculatorType): ComputeNodeType {
        switch (calc) {
            case CalculatorType.SectionParametree:
                if (sect === "select_section_trapez")
                    return ComputeNodeType.SectionTrapeze;
                if (sect === "select_section_rect")
                    return ComputeNodeType.SectionRectangle;
                if (sect === "select_section_circ")
                    return ComputeNodeType.SectionCercle;
                if (sect === "select_section_puiss")
                    return ComputeNodeType.SectionPuissance;
                throw "Formulaire.getSectionType() : type de section '" + sect + "' inconnu pour la calculette " + CalculatorType[calc];

            case CalculatorType.CourbeRemous:
                if (sect === "select_section_trapez")
                    return ComputeNodeType.CourbeRemousTrapeze;
                if (sect === "select_section_rect")
                    return ComputeNodeType.CourbeRemousRectangle;
                if (sect === "select_section_circ")
                    return ComputeNodeType.CourbeRemousCercle;
                if (sect === "select_section_puiss")
                    return ComputeNodeType.CourbeRemousPuissance;
                throw "Formulaire.getSectionType() : type de section '" + sect + "' inconnu pour la calculette " + CalculatorType[calc];

            default:
                throw "Formulaire.getSectionType() : type de calculette " + CalculatorType[calc] + " non pris en charge";
        }
    }

    public isDisplayed(id: string) {
        return this.getFormulaireElementById(id).isDisplayed;
    }
}

export abstract class FormulaireElement {
    private _nodeType: ComputeNodeType;
    private _id: string;
    public isDisplayed: boolean;
    public label: string;

    constructor(nodeType: ComputeNodeType, id: string) {
        this._nodeType = nodeType;
        this._id = id;
        this.isDisplayed = true;
    }

    get computeNodeType(): ComputeNodeType {
        return this._nodeType;
    }

    get id(): string {
        return this._id;
    }

    protected abstract verifyDependency(d: Dependency): boolean;

    public verifiesDependency(d: Dependency): boolean {
        if (d.masterCondition.type == DependencyConditionType.IsDisplayed)
            return this.isDisplayed;

        return this.verifyDependency(d);
    }

    public abstract updateLocalisation(loc: StringMap): void;

    public toString() {
        return "id:" + this._id + (this.isDisplayed ? " displayed" : " NOT displayed") + " label:" + this.label;
    }
}

export enum FieldType {
    Input, Select, Check
}

export abstract class Field extends FormulaireElement {
    constructor(nodeType: ComputeNodeType, id: string, private _fieldType: FieldType) {
        super(nodeType, id);
    }

    public get isInput(): boolean {
        return this._fieldType == FieldType.Input;
    }

    public get isSelect(): boolean {
        return this._fieldType == FieldType.Select;
    }

    public get isCheck(): boolean {
        return this._fieldType == FieldType.Check;
    }

    public abstract getValue(): any;
    public abstract setValue(val: any): void;
}

export class FieldSet extends FormulaireElement {
    private _fields: Field[];

    constructor(nodeType: ComputeNodeType, id: string) {
        super(nodeType, id);
        this._fields = [];
    }

    public get fields() {
        return this._fields;
    }

    public addField(f: Field) {
        this._fields.push(f);
    }

    public get hasInputs(): boolean {
        for (let f of this._fields)
            if (f instanceof NgParameter)
                return true;
        return false;
    }

    public getInput(i: number): NgParameter {
        let n = 0;
        for (let f of this._fields) {
            if (f instanceof NgParameter) {
                if (n == i)
                    return f;
                n++;
            }
        }
        return undefined;
    }

    protected verifyDependency(d: Dependency): boolean {
        throw "FieldSet.verifyDependency() : type de condition '" + DependencyConditionType[d.masterCondition.type] + "' non pris en charge";
    }

    public updateLocalisation(loc: StringMap) {
        this.label = loc[this.id];
    }
}

export class SelectEntry {
    private _value: string;
    public label: string;

    constructor(v: string, l: string) {
        this._value = v;
        this.label = l;
    }

    get value(): string {
        return this._value;
    }
}

export class SelectField extends Field {
    private _entries: SelectEntry[];

    public selectedEntry: SelectEntry;

    public get entries() {
        return this._entries;
    }

    constructor(nodeType: ComputeNodeType, id: string) {
        super(nodeType, id, FieldType.Select);
        this._entries = [];
    }

    public addEntry(e: SelectEntry) {
        this._entries.push(e);
        if (this.selectedEntry == undefined)
            this.selectedEntry = e;
    }

    public getValue() {
        if (this.selectedEntry == undefined)
            return undefined;
        return this.selectedEntry.value;
    }

    public setValue(val: string) {
        for (let e of this._entries)
            if (e.value === val) {
                this.selectedEntry = e;
                return;
            }
    }

    public getLabel() {
        if (this.selectedEntry == undefined)
            return undefined;
        return this.selectedEntry.label;
    }

    protected verifyDependency(d: Dependency): boolean {
        switch (d.masterCondition.type) {
            case DependencyConditionType.HasValue:
                let mc: ValueDependencyCondition = <ValueDependencyCondition>d.masterCondition;
                return this.selectedEntry.value === mc.value;

            default:
                throw "SelectField.verifyDependency() : type de condition '" + DependencyConditionType[d.masterCondition.type] + "' non pris en charge";
        }
    }

    public updateLocalisation(loc: StringMap) {
        this.label = loc[this.id];
        for (let e of this._entries) {
            e.label = loc[e.value];
        }
    }
}

export class CheckField extends Field {
    private _value: boolean;

    constructor(nodeType: ComputeNodeType, id: string) {
        super(nodeType, id, FieldType.Check);
        this._value = false;
    }

    public getValue(): boolean {
        return this._value;
    }

    public setValue(val: boolean) {
        this._value = val;
    }

    protected verifyDependency(d: Dependency): boolean {
        throw "CheckField.verifyDependency() : type de condition '" + DependencyConditionType[d.masterCondition.type] + "' non pris en charge";
    }

    public updateLocalisation(loc: StringMap) {
        this.label = loc[this.id];
    }
}

export abstract class InputField extends Field {
    private _value: any;

    constructor(type: ComputeNodeType, id: string) {
        super(type, id, FieldType.Input);
    }

    public getValue() {
        return this._value;
    }

    public setValue(val: any) {
        this._value = val;
    }

    public updateLocalisation(loc: StringMap) {
        this.label = loc[this.id];
    }
}

export enum DependencyConditionType {
    HasValue, IsVariable, IsDisplayed
}

export class DependencyCondition {
    constructor(private _type: DependencyConditionType) {
    }

    public get type() {
        return this._type;
    }

    public toString(): string {
        return "cond=" + DependencyConditionType[this._type];
    }
}

export class ValueDependencyCondition extends DependencyCondition {
    constructor(private _value: any) {
        super(DependencyConditionType.HasValue);
    }

    public get value(): any {
        return this._value;
    }

    public toString(): string {
        return super.toString() + " " + this._value;
    }
}

export abstract class Dependency {
    private _master: FormulaireElement;

    private _slave: FormulaireElement;

    private _masterCondition: DependencyCondition;

    constructor(m: FormulaireElement, s: FormulaireElement, mc: DependencyCondition) {
        this._master = m;
        this._slave = s;
        this._masterCondition = mc;
    }

    public get masterElement(): FormulaireElement {
        return this._master;
    }

    public get slaveElement(): FormulaireElement {
        return this._slave;
    }

    public get masterCondition(): DependencyCondition {
        return this._masterCondition;
    }

    public toString(): string {
        return "master=" + this._master.toString() + "\n  " + this._masterCondition.toString() + "\n  slave=" + this._slave.toString();
    }
}

export class ValueDependency extends Dependency {
    public slaveValue: any;

    constructor(m: FormulaireElement, s: FormulaireElement, masterValue: any) {
        super(m, s, new ValueDependencyCondition(masterValue));
    }

    public toString() {
        return "valdep\n  " + super.toString() + "\n  slave val " + this.slaveValue;
    }
}

export class ExistenceDependency extends Dependency {
    /**
     * true : l'élément slave est affiché si le master est affiché
     * false : l'élément slave est affiché si le master n'est pas affiché
     */
    // private _direct: boolean;

    public toString() {
        return "existdep\n  " + super.toString();
    }
}
