import { Injectable } from '@angular/core';
import { Response } from '@angular/http';

import { ComputeNodeType, ParamsEquation, acSection, Nub, ConduiteDistrib, ConduiteDistribParams } from "jalhyd";
import { LechaptCalmon, LechaptCalmonParams, ParamsSectionTrapez, cSnTrapez } from "jalhyd";
import { ParamsSectionRectang, cSnRectang, ParamsSectionCirc, cSnCirc, ParamsSectionPuiss, cSnPuiss } from "jalhyd";

import { ParamService } from '../param/param.service';
import { HttpService } from '../../services/http/http.service';
import { FormulaireDefinition, FormulaireElement, CalculatorType, SelectField, CheckField } from '../../calculators/generic/formulaire';
import { StringMap } from '../../stringmap';

@Injectable()
export class FormulaireService {
    private _formulaires: FormulaireDefinition[];

    constructor(private paramService: ParamService, private httpService: HttpService) {
        this._formulaires = [];
    }

    private loadConfig(form: FormulaireDefinition, ct: CalculatorType): Promise<Response> {
        let processData = function (s: string) {
            form.parseConfig(JSON.parse(s));
        }

        let f: string = this.getConfigPathPrefix(ct) + "config.json"
        return this.httpService.httpGetRequest(undefined, undefined, undefined, f, processData);
    }

    public loadFormulaire(ct: CalculatorType): Promise<FormulaireDefinition> {
        if (ct == undefined)
            throw "FormulaireService.getFormulaire() : invalid undefined CalculatorType"

        let f = new FormulaireDefinition(this.paramService, ct);
        this._formulaires.push(f);
        let pr: Promise<FormulaireDefinition> = this.loadConfig(f, ct)
            .then(res => {
                return f;
            });
        return pr;
    }

    private getFormulaire(ct: CalculatorType): FormulaireDefinition {
        if (ct == undefined)
            throw "FormulaireService.getFormulaire() : invalid undefined CalculatorType"

        for (let f of this._formulaires)
            if (f.type == ct)
                return f;

        throw "FormulaireService.getFormulaire() : type '" + ct + "' form is not loaded";
    }

    private getFormulaireElementById(id: string): FormulaireElement {
        for (let f of this._formulaires) {
            let s = f.getFormulaireElementById(id);
            if (s != undefined)
                return s;
        }
        return undefined;
    }

    public getSelectField(id: string): SelectField {
        for (let f of this._formulaires) {
            let s = f.getFormulaireElementById(id);
            if (s != undefined)
                if (!(s instanceof SelectField))
                    throw "Form element with id '" + id + "' is not a select";
            return <SelectField>s;
        }
        return undefined;
    }

    public getCheckField(id: string): CheckField {
        for (let f of this._formulaires) {
            let s = f.getFormulaireElementById(id);
            if (s != undefined)
                if (!(s instanceof CheckField))
                    throw "Form element with id '" + id + "' is not a checkbox";
            return <CheckField>s;
        }
        return undefined;
    }

    public updateLocalisation(localisation: StringMap) {
        for (let loc_id in localisation) {
            let fe = this.getFormulaireElementById(loc_id);
            if (fe != undefined)
                fe.updateLocalisation(localisation);
        }
    }

    public getNubAndParameters(ct: CalculatorType): [Nub, ParamsEquation] {
        let f = this.getFormulaire(ct);
        switch (ct) {
            case CalculatorType.ConduiteDistributrice:
                {
                    let Q: number = f.getParameterValue("Q"); // débit Q
                    let D: number = f.getParameterValue("D"); // diamètre D
                    let J: number = f.getParameterValue("J"); // perte de charge J
                    let Lg: number = f.getParameterValue("Lg"); // Longueur de la conduite Lg
                    let Nu: number = f.getParameterValue("Nu"); // viscosité dynamique 
                    let prms = new ConduiteDistribParams(Q, D, J, Lg, Nu);
                    let nub = new ConduiteDistrib(prms);
                    return [nub, prms];
                }

            case CalculatorType.LechaptCalmon:
                {
                    let Q: number = f.getParameterValue("Q"); // débit Q
                    let D: number = f.getParameterValue("D"); // diamètre D
                    let J: number = f.getParameterValue("J"); // perte de charge J
                    let Lg: number = f.getParameterValue("Lg"); // Longueur de la conduite Lg
                    let L: number = f.getParameterValue("L"); // paramètre de matériau 1
                    let M: number = f.getParameterValue("M"); // paramètre de matériau 2
                    let N: number = f.getParameterValue("N"); // paramètre de matériau 3
                    let prms = new LechaptCalmonParams(Q, D, J, Lg, L, M, N);
                    let nub = new LechaptCalmon(prms);
                    return [nub, prms];
                }

            default:
                throw "FormulaireService.getNubAndParameters() : valeur de CalculatorType " + ct + " non implémentée"
        }
    }

    public getSectionNubAndParameters(ct: CalculatorType, nt: ComputeNodeType, getY: boolean = true): [acSection, ParamsEquation] {
        let f: FormulaireDefinition = this.getFormulaire(ct);

        // bief
        // let Ks = f.getNodeParameterValue(nt, "Ks"); // Strickler
        let Ks = f.getParameterValue("Ks"); // Strickler
        let If: number = f.getParameterValue("If"); // Pente du fond
        let YB: number = f.getParameterValue("YB"); // Hauteur de berge

        // caractéristiques hydro
        let Q: number = f.getParameterValue("Q"); // débit Q
        if (getY)
            var Y: number = f.getParameterValue("Y"); // tirant d'eau
        else
            Y = undefined;

        let Prec = f.getParameterValue("Pr"); // précision calcul/affichage

        // ??
        // let YCL: number = f.getParameterValue("YCL"); // Condition limite en cote à l'amont ou à l'aval
        // let Dx: number = f.getParameterValue("Dx"); // Pas d'espace (positif en partant de l'aval, négatif en partant de l'amont)
        // let Long: number = f.getParameterValue("Long"); // Longueur du bief

        switch (nt) {
            case ComputeNodeType.SectionTrapeze:
            case ComputeNodeType.RegimeUniformeTrapeze:
            case ComputeNodeType.CourbeRemousTrapeze:
                {
                    let LargeurFond = f.getNodeParameterValue(nt, "LargeurFond"); // Largeur au fond
                    let Fruit = f.getNodeParameterValue(nt, "Fruit"); // Fruit des berges
                    let prms = new ParamsSectionTrapez(LargeurFond, Fruit, Y, Ks, Q, If, Prec, YB);
                    let cn = new cSnTrapez(undefined, prms);
                    return [cn, prms];
                }

            case ComputeNodeType.SectionRectangle:
            case ComputeNodeType.RegimeUniformeRectangle:
            case ComputeNodeType.CourbeRemousRectangle:
                {
                    let LargeurFond = f.getNodeParameterValue(nt, "LargeurBerge"); // Largeur au fond
                    let prms = new ParamsSectionRectang(Y, LargeurFond, Ks, Q, If, Prec, YB);
                    let cn = new cSnRectang(undefined, prms);
                    return [cn, prms];
                }

            case ComputeNodeType.SectionCercle:
            case ComputeNodeType.RegimeUniformeCercle:
            case ComputeNodeType.CourbeRemousCercle:
                {
                    let D = f.getNodeParameterValue(nt, "D"); // Largeur au fond
                    let prms = new ParamsSectionCirc(D, Y, Ks, Q, If, Prec, YB);
                    let cn = new cSnCirc(undefined, prms);
                    return [cn, prms];
                }

            case ComputeNodeType.SectionPuissance:
            case ComputeNodeType.RegimeUniformePuissance:
            case ComputeNodeType.CourbeRemousPuissance:
                {
                    let k = f.getNodeParameterValue(nt, "k"); // coefficient
                    let LargeurBerge = f.getNodeParameterValue(nt, "LargeurBerge"); // Largeur au niveau des berges
                    let prms = new ParamsSectionPuiss(k, Y, LargeurBerge, Ks, Q, If, Prec, YB);
                    let cn = new cSnPuiss(undefined, prms);
                    return [cn, prms];
                }

            default:
                throw "FormulaireService.getSectionNubAndParameters() : valeur de SectionType " + nt + " non implémentée"
        }
    }

    public getConfigPathPrefix(ct: CalculatorType): string {
        if (ct == undefined)
            throw "FormulaireService.getConfigPathPrefix() : invalid undefined CalculatorType"

        switch (ct) {
            case CalculatorType.ConduiteDistributrice:
                return "app/calculators/cond_distri/cond_distri.";

            case CalculatorType.LechaptCalmon:
                return "app/calculators/lechapt-calmon/lechapt-calmon.";

            case CalculatorType.SectionParametree:
                return "app/calculators/section-param/section-param.";

            case CalculatorType.RegimeUniforme:
                return "app/calculators/regime-uniforme/regime-uniforme.";

            case CalculatorType.CourbeRemous:
                return "app/calculators/remous/remous.";

            default:
                throw "FormulaireService.getConfigPathPrefix() : valeur de CalculatorType " + ct + " non implémentée"
        }
    }
}
